==> fetch.php?media=resources_multi_hardware:lorafwd_1.0.7.tar.gz: lorafwd_1.0.7/config.cc <==
// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "config.h"

#include <iomanip>
#include <memory>
#include <sstream>

#include "third-party/cpptoml.h"

#include "utils/logger.h"
#include "utils/stream.h"

#ifndef RUNSTATEDIR
#define RUNSTATEDIR "/var/run/lora"
#endif

#ifdef DEPRECATED_FEATURES
const std::size_t Config::GWMP_AUTORESTART_FALSE;
const std::size_t Config::GWMP_AUTORESTART_TRUE;
#endif

Config::Config(void)
	: gatewayId(0), filterCRCValid(true), filterCRCInvalid(false),
	  filterCRCNone(false), dbEnable(false), dbLimitMessages(200),
	  dbDelayCooldown(100),
#ifdef DEPRECATED_FEATURES
	  gwmpNode("localhost"), gwmpServiceUplink("0"),
	  gwmpServiceDownlink("0"), gwmpPeriodHeartbeat(10),
	  gwmpPeriodStatistics(30), gwmpAutorestart(GWMP_AUTORESTART_TRUE),
	  gwmpLimitDatagram(20000),
	  gwmpLoradControl("ipc://" RUNSTATEDIR "/lorad"),
#endif
	  apiUplink({"ipc://" RUNSTATEDIR "/uplink"}),
	  apiDownlink({"ipc://" RUNSTATEDIR "/downlink"}),
	  apiControl({"ipc://" RUNSTATEDIR "/lorafwd"}), apiFilters({""})
{
}

namespace {
template <typename T, typename U>
std::ostream &operator<<(std::ostream &_os,
			 const utils::StrongType<T, U> &_value)
{
	return (_os << static_cast<T>(_value));
}

std::ostream &operator<<(std::ostream &_os, Config::bool_type _value)
{
	return (_os << ((_value) ? "true" : "false"));
}

std::ostream &operator<<(std::ostream &_os, const Config::gwid_type &_value)
{
	return (_os << utils::to_hex(static_cast<std::uint64_t>(_value)));
}

std::ostream &operator<<(std::ostream &_os,
			 const std::chrono::milliseconds &_value)
{
	return (_os << _value.count() << " msec");
}

#ifdef DEPRECATED_FEATURES
std::ostream &operator<<(std::ostream &_os,
			 const Config::gwmp_autorestart_type &_value)
{
	if (_value) {
		return (_os << static_cast<std::size_t>(_value));
	} else {
		return (_os << static_cast<Config::bool_type>(_value));
	}
}

std::ostream &operator<<(std::ostream &_os, const std::chrono::seconds &_value)
{
	return (_os << _value.count() << " sec");
}
#endif
}

namespace {
template <typename T>
void log_value(const std::string &_key, const T &_value)
{
	logr::debug << "Configuring \"" << _key << "\" with " << _value
		    << logr::endl;
}

using cpptoml_type = std::shared_ptr<cpptoml::table>;

template <typename T>
void parse_value(cpptoml_type _config, const std::string &_key, T &_value)
{
	if (auto value = _config->get_qualified_as<T>(_key)) {
		_value = *value;

		log_value(_key, _value);
	}
}

template <typename T, typename U>
void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 utils::StrongType<T, U> &_value)
{
	if (auto value = _config->get_qualified_as<T>(_key)) {
		_value = *value;

		log_value(_key, _value);
	}
}

void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 Config::gwid_type &_value)
{
	if (auto value = _config->get_qualified_as<std::uint64_t>(_key)) {
		_value = *value;

		log_value(_key, _value);

	} else if (auto value = _config->get_qualified_as<std::string>(_key)) {
		std::uint64_t raw;
		std::istringstream string(*value);
		string >> std::hex >> raw;
		_value = raw;

		log_value(_key, _value);
	}
}

template <class Rep, class Period>
void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 std::chrono::duration<Rep, Period> &_value)
{
	if (auto value = _config->get_qualified_as<std::size_t>(_key)) {
		_value = std::chrono::duration<Rep, Period>(*value);

		log_value(_key, _value);
	}
}

#ifdef DEPRECATED_FEATURES
void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 Config::gwmp_service_type &_value)
{
	if (auto value = _config->get_qualified_as<std::string>(_key)) {
		_value = *value;

		log_value(_key, _value);

	} else if (auto value =
			   _config->get_qualified_as<std::uint32_t>(_key)) {
		_value = std::to_string(*value);

		log_value(_key, _value);
	}
}

void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 Config::gwmp_autorestart_type &_value)
{
	if (auto value = _config->get_qualified_as<std::size_t>(_key)) {
		_value = *value;

		log_value(_key, _value);

	} else if (auto value = _config->get_qualified_as<bool>(_key)) {
		_value = (*value) ? Config::GWMP_AUTORESTART_TRUE
				  : Config::GWMP_AUTORESTART_FALSE;

		log_value(_key, _value);
	}
}
#endif

template <typename T>
void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 std::vector<T> &_values)
{
	if (auto values = _config->get_qualified_array_of<T>(_key)) {
		_values.clear();

		for (const auto &value : *values) {
			_values.push_back(value);
		}

		log_value(_key, _values);

	} else if (auto value = _config->get_qualified_as<T>(_key)) {
		_values.clear();
		_values.push_back(*value);

		log_value(_key, _values);
	}
}

template <typename T>
void parse(cpptoml_type _config, const std::string &_key, T &_value)
{
	try {
		parse_value(_config, _key, _value);

	} catch (const std::exception &e) {
		logr::error << "Failed to parse \"" << _key
			    << "\": " << e.what() << logr::endl;
		throw 0;
	}
}
}

bool Config::parse_file(const std::string &_file)
{
	try {
		auto config = cpptoml::parse_file(_file);

		parse(config, "gateway.id", gatewayId);

		parse(config, "filter.crc.valid", filterCRCValid);
		parse(config, "filter.crc.invalid", filterCRCInvalid);
		parse(config, "filter.crc.none", filterCRCNone);

		parse(config, "database.enable", dbEnable);
		parse(config, "database.limit.messages", dbLimitMessages);
		parse(config, "database.delay.cooldown", dbDelayCooldown);

#ifdef DEPRECATED_FEATURES
		parse(config, "gwmp.node", gwmpNode);
		parse(config, "gwmp.service.uplink", gwmpServiceUplink);
		parse(config, "gwmp.service.downlink", gwmpServiceDownlink);
		parse(config, "gwmp.period.heartbeat", gwmpPeriodHeartbeat);
		parse(config, "gwmp.period.statistics", gwmpPeriodStatistics);
		parse(config, "gwmp.autorestart", gwmpAutorestart);
		parse(config, "gwmp.limit.datagram", gwmpLimitDatagram);
		parse(config, "gwmp.lorad.control", gwmpLoradControl);
#endif

		parse(config, "api.uplink", apiUplink);
		parse(config, "api.downlink", apiDownlink);
		parse(config, "api.control", apiControl);
		parse(config, "api.filters", apiFilters);

		return true;

	} catch (const std::exception &e) {
		logr::error << "Failed to parse file: " << e.what()
			    << logr::endl;
		return false;

	} catch (...) {
		return false;
	}
}

namespace {
template <typename T>
struct Dump
{
	std::string key;
	T value;
};

template <typename T>
Dump<T> dump(const std::string &_key, const T &_value)
{
	return {_key, _value};
}

template <typename T>
std::ostream &operator<<(std::ostream &_os, const Dump<T> &_dump)
{
	_os << " " << std::left << std::setw(40) << _dump.key;
	return (_os << ": " << _dump.value << std::endl);
}
}

std::ostream &operator<<(std::ostream &_os, const Config &_config)
{
	_os << "Using configuration:" << std::endl;

	_os << dump("gateway.id", _config.gatewayId);

	_os << dump("filter.crc.valid", _config.filterCRCValid);
	_os << dump("filter.crc.invalid", _config.filterCRCInvalid);
	_os << dump("filter.crc.none", _config.filterCRCNone);

	_os << dump("database.enable", _config.dbEnable);
	_os << dump("database.limit.messages", _config.dbLimitMessages);
	_os << dump("database.delay.cooldown", _config.dbDelayCooldown);

#ifdef DEPRECATED_FEATURES
	_os << dump("gwmp.node", _config.gwmpNode);
	_os << dump("gwmp.service.uplink", _config.gwmpServiceUplink);
	_os << dump("gwmp.service.downlink", _config.gwmpServiceDownlink);
	_os << dump("gwmp.period.heartbeat", _config.gwmpPeriodHeartbeat);
	_os << dump("gwmp.period.statistics", _config.gwmpPeriodStatistics);
	_os << dump("gwmp.autorestart", _config.gwmpAutorestart);
	_os << dump("gwmp.limit.datagram", _config.gwmpLimitDatagram);
	_os << dump("gwmp.lorad.control", _config.gwmpLoradControl);
#endif

	_os << dump("api.uplink", _config.apiUplink);
	_os << dump("api.downlink", _config.apiDownlink);
	_os << dump("api.control", _config.apiControl);
	_os << dump("api.filters", _config.apiFilters);

	return _os;
}

==> fetch.php?media=resources_multi_hardware:lorafwd_1.1.1.tar.gz: lorafwd_1.1.1/config.cc <==
// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "config.h"

#include <iomanip>
#include <memory>
#include <sstream>

#include "third-party/cpptoml.h"

#include "utils/logger.h"
#include "utils/stream.h"

#ifndef RUNSTATEDIR
#define RUNSTATEDIR "/var/run/lora"
#endif

#ifdef DEPRECATED_FEATURES
const std::size_t Config::GWMP_AUTORESTART_FALSE;
const std::size_t Config::GWMP_AUTORESTART_TRUE;
#endif

Config::Config(void)
	: gatewayId(0), filterCRCValid(true), filterCRCInvalid(false),
	  filterCRCNone(false), filterLoRaWANDownlink(false), dbEnable(false),
	  dbLimitMessages(200), dbDelayCooldown(100),
#ifdef DEPRECATED_FEATURES
	  gwmpNode("localhost"), gwmpServiceUplink("0"),
	  gwmpServiceDownlink("0"), gwmpPeriodHeartbeat(10),
	  gwmpPeriodStatistics(30), gwmpAutorestart(GWMP_AUTORESTART_TRUE),
	  gwmpLimitDatagram(20000),
	  gwmpLoradControl("ipc://" RUNSTATEDIR "/lorad"),
#endif
	  apiUplink({"ipc://" RUNSTATEDIR "/uplink"}),
	  apiDownlink({"ipc://" RUNSTATEDIR "/downlink"}),
	  apiControl({"ipc://" RUNSTATEDIR "/lorafwd"}), apiFilters({""})
{
}

namespace {
template <typename T, typename U>
std::ostream &operator<<(std::ostream &_os,
			 const utils::StrongType<T, U> &_value)
{
	return (_os << static_cast<T>(_value));
}

std::ostream &operator<<(std::ostream &_os, Config::bool_type _value)
{
	return (_os << ((_value) ? "true" : "false"));
}

std::ostream &operator<<(std::ostream &_os, const Config::gwid_type &_value)
{
	return (_os << utils::to_hex(static_cast<std::uint64_t>(_value)));
}

std::ostream &operator<<(std::ostream &_os,
			 const std::chrono::milliseconds &_value)
{
	return (_os << _value.count() << " msec");
}

#ifdef DEPRECATED_FEATURES
std::ostream &operator<<(std::ostream &_os,
			 const Config::gwmp_autorestart_type &_value)
{
	if (_value) {
		return (_os << static_cast<std::size_t>(_value));
	} else {
		return (_os << static_cast<Config::bool_type>(_value));
	}
}

std::ostream &operator<<(std::ostream &_os, const std::chrono::seconds &_value)
{
	return (_os << _value.count() << " sec");
}
#endif
}

namespace {
template <typename T>
void log_value(const std::string &_key, const T &_value)
{
	logr::debug << "Configuring \"" << _key << "\" with " << _value
		    << logr::endl;
}

using cpptoml_type = std::shared_ptr<cpptoml::table>;

template <typename T>
void parse_value(cpptoml_type _config, const std::string &_key, T &_value)
{
	if (auto value = _config->get_qualified_as<T>(_key)) {
		_value = *value;

		log_value(_key, _value);
	}
}

template <typename T, typename U>
void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 utils::StrongType<T, U> &_value)
{
	if (auto value = _config->get_qualified_as<T>(_key)) {
		_value = *value;

		log_value(_key, _value);
	}
}

void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 Config::gwid_type &_value)
{
	if (auto value = _config->get_qualified_as<std::uint64_t>(_key)) {
		_value = *value;

		log_value(_key, _value);

	} else if (auto value = _config->get_qualified_as<std::string>(_key)) {
		std::uint64_t raw;
		std::istringstream string(*value);
		string >> std::hex >> raw;
		_value = raw;

		log_value(_key, _value);
	}
}

template <class Rep, class Period>
void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 std::chrono::duration<Rep, Period> &_value)
{
	if (auto value = _config->get_qualified_as<std::size_t>(_key)) {
		_value = std::chrono::duration<Rep, Period>(*value);

		log_value(_key, _value);
	}
}

#ifdef DEPRECATED_FEATURES
void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 Config::gwmp_service_type &_value)
{
	if (auto value = _config->get_qualified_as<std::string>(_key)) {
		_value = *value;

		log_value(_key, _value);

	} else if (auto value =
			   _config->get_qualified_as<std::uint32_t>(_key)) {
		_value = std::to_string(*value);

		log_value(_key, _value);
	}
}

void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 Config::gwmp_autorestart_type &_value)
{
	if (auto value = _config->get_qualified_as<std::size_t>(_key)) {
		_value = *value;

		log_value(_key, _value);

	} else if (auto value = _config->get_qualified_as<bool>(_key)) {
		_value = (*value) ? Config::GWMP_AUTORESTART_TRUE
				  : Config::GWMP_AUTORESTART_FALSE;

		log_value(_key, _value);
	}
}
#endif

template <typename T>
void parse_value(cpptoml_type _config,
		 const std::string &_key,
		 std::vector<T> &_values)
{
	if (auto values = _config->get_qualified_array_of<T>(_key)) {
		_values.clear();

		for (const auto &value : *values) {
			_values.push_back(value);
		}

		log_value(_key, _values);

	} else if (auto value = _config->get_qualified_as<T>(_key)) {
		_values.clear();
		_values.push_back(*value);

		log_value(_key, _values);
	}
}

template <typename T>
void parse(cpptoml_type _config, const std::string &_key, T &_value)
{
	try {
		parse_value(_config, _key, _value);

	} catch (const std::exception &e) {
		logr::error << "Failed to parse \"" << _key
			    << "\": " << e.what() << logr::endl;
		throw 0;
	}
}
}

bool Config::parse_file(const std::string &_file)
{
	try {
		auto config = cpptoml::parse_file(_file);

		parse(config, "gateway.id", gatewayId);

		parse(config, "filter.crc.valid", filterCRCValid);
		parse(config, "filter.crc.invalid", filterCRCInvalid);
		parse(config, "filter.crc.none", filterCRCNone);
		parse(config, "filter.lorawan.downlink", filterLoRaWANDownlink);

		parse(config, "database.enable", dbEnable);
		parse(config, "database.limit.messages", dbLimitMessages);
		parse(config, "database.delay.cooldown", dbDelayCooldown);

#ifdef DEPRECATED_FEATURES
		parse(config, "gwmp.node", gwmpNode);
		parse(config, "gwmp.service.uplink", gwmpServiceUplink);
		parse(config, "gwmp.service.downlink", gwmpServiceDownlink);
		parse(config, "gwmp.period.heartbeat", gwmpPeriodHeartbeat);
		parse(config, "gwmp.period.statistics", gwmpPeriodStatistics);
		parse(config, "gwmp.autorestart", gwmpAutorestart);
		parse(config, "gwmp.limit.datagram", gwmpLimitDatagram);
		parse(config, "gwmp.lorad.control", gwmpLoradControl);
#endif

		parse(config, "api.uplink", apiUplink);
		parse(config, "api.downlink", apiDownlink);
		parse(config, "api.control", apiControl);
		parse(config, "api.filters", apiFilters);

		return true;

	} catch (const std::exception &e) {
		logr::error << "Failed to parse file: " << e.what()
			    << logr::endl;
		return false;

	} catch (...) {
		return false;
	}
}

namespace {
template <typename T>
struct Dump
{
	std::string key;
	T value;
};

template <typename T>
Dump<T> dump(const std::string &_key, const T &_value)
{
	return {_key, _value};
}

template <typename T>
std::ostream &operator<<(std::ostream &_os, const Dump<T> &_dump)
{
	_os << " " << std::left << std::setw(40) << _dump.key;
	return (_os << ": " << _dump.value << std::endl);
}
}

std::ostream &operator<<(std::ostream &_os, const Config &_config)
{
	_os << "Using configuration:" << std::endl;

	_os << dump("gateway.id", _config.gatewayId);

	_os << dump("filter.crc.valid", _config.filterCRCValid);
	_os << dump("filter.crc.invalid", _config.filterCRCInvalid);
	_os << dump("filter.crc.none", _config.filterCRCNone);
	_os << dump("filter.lorawan.downlink", _config.filterLoRaWANDownlink);

	_os << dump("database.enable", _config.dbEnable);
	_os << dump("database.limit.messages", _config.dbLimitMessages);
	_os << dump("database.delay.cooldown", _config.dbDelayCooldown);

#ifdef DEPRECATED_FEATURES
	_os << dump("gwmp.node", _config.gwmpNode);
	_os << dump("gwmp.service.uplink", _config.gwmpServiceUplink);
	_os << dump("gwmp.service.downlink", _config.gwmpServiceDownlink);
	_os << dump("gwmp.period.heartbeat", _config.gwmpPeriodHeartbeat);
	_os << dump("gwmp.period.statistics", _config.gwmpPeriodStatistics);
	_os << dump("gwmp.autorestart", _config.gwmpAutorestart);
	_os << dump("gwmp.limit.datagram", _config.gwmpLimitDatagram);
	_os << dump("gwmp.lorad.control", _config.gwmpLoradControl);
#endif

	_os << dump("api.uplink", _config.apiUplink);
	_os << dump("api.downlink", _config.apiDownlink);
	_os << dump("api.control", _config.apiControl);
	_os << dump("api.filters", _config.apiFilters);

	return _os;
}
